#include "adc.h"
#include "FreeRTOS.h"
#include "task.h"

void ADC_Init(void)
{
    FL_GPIO_InitTypeDef    GPIO_InitStruct;
		FL_ADC_CommonInitTypeDef    CommonInitStruct;
    FL_ADC_InitTypeDef    defaultInitStruct;
		
    CommonInitStruct.clockSource = FL_RCC_ADC_CLK_SOURCE_RCHF;
    CommonInitStruct.clockPrescaler = FL_RCC_ADC_PSC_DIV8;

    FL_ADC_CommonInit(&CommonInitStruct );

    GPIO_InitStruct.pin = FL_GPIO_PIN_0;
    GPIO_InitStruct.mode = FL_GPIO_MODE_ANALOG;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull = FL_DISABLE;
    GPIO_InitStruct.remapPin = FL_DISABLE;

    FL_GPIO_Init( GPIOD, &GPIO_InitStruct );

    defaultInitStruct.conversionMode = FL_ADC_CONV_MODE_SINGLE;
    defaultInitStruct.autoMode = FL_ADC_SINGLE_CONV_MODE_AUTO;
    defaultInitStruct.waitMode = FL_ENABLE;
    defaultInitStruct.overrunMode = FL_ENABLE;
    defaultInitStruct.scanDirection = FL_ADC_SEQ_SCAN_DIR_BACKWARD;
    defaultInitStruct.externalTrigConv = FL_ADC_TRIGGER_EDGE_NONE;
    defaultInitStruct.triggerSource = FL_ADC_TRGI_PA8;
    defaultInitStruct.fastChannelTime = FL_ADC_FAST_CH_SAMPLING_TIME_4_ADCCLK;
    defaultInitStruct.lowChannelTime = FL_ADC_SLOW_CH_SAMPLING_TIME_192_ADCCLK;
    defaultInitStruct.oversamplingMode = FL_ENABLE;
    defaultInitStruct.overSampingMultiplier = FL_ADC_OVERSAMPLING_MUL_16X;
    defaultInitStruct.oversamplingShift = FL_ADC_OVERSAMPLING_SHIFT_4B;

    FL_ADC_Init(ADC,&defaultInitStruct );

}

static uint32_t GetAVREFSample_POLL(uint32_t* ADCRdresult)
{   
    uint32_t counter = 0;
    uint32_t EOC_Flag;
    uint8_t State;
    FL_RCC_SetADCPrescaler(FL_RCC_ADC_PSC_DIV8);                    //配置ADC工作时钟分频
    FL_VREF_EnableVREFBuffer(VREF);                                 //使能VREF BUFFER,
                                                                    //为了考虑功耗BUFFER可关闭，从使能到ADC可以正常采样需要100us的建立时间，本例程配置慢速通道较长的采样时间，来保证建立时间满足。
    FL_ADC_DisableSequencerChannel(ADC, FL_ADC_ALL_CHANNEL);        //清空打开的通道
    FL_ADC_EnableSequencerChannel(ADC, FL_ADC_INTERNAL_AVREF);    //通道选择

    FL_ADC_ClearFlag_EndOfConversion(ADC);                          //清标志
    FL_ADC_Enable(ADC);                                             //启动ADC
    FL_ADC_EnableSWConversion(ADC);                                 //开始转换

    do
    {
        EOC_Flag = FL_ADC_IsActiveFlag_EndOfConversion(ADC);
        counter++;
    }while((counter != 0xFFFFFFFFU) && (EOC_Flag == 0U));           //等待转换完成

    if(EOC_Flag == 0x01U)
    {
      FL_ADC_ClearFlag_EndOfConversion(ADC);                        //清标志
      *ADCRdresult = FL_ADC_ReadConversionData(ADC);                 //获取采样值

      FL_ADC_Disable(ADC);                                          //关闭ADC
      FL_ADC_DisableSequencerChannel(ADC, FL_ADC_INTERNAL_AVREF); //通道关闭
      FL_VREF_DisableVREFBuffer(VREF);                              //关闭VREF BUFFER
      State = 0;
    }
    else
    {
      State = 1 ;
    }

    return State;
}


static uint32_t GetSingleChannelSample_POLL(uint32_t channel,uint32_t* ADCRdresult)
{
    uint32_t counter = 0; 
    uint32_t EOC_Flag;
    uint8_t State;

    FL_RCC_SetADCPrescaler(FL_RCC_ADC_PSC_DIV1);                  //配置ADC工作时钟分频
    FL_ADC_DisableSequencerChannel(ADC, FL_ADC_ALL_CHANNEL);      //清空打开的通道
    FL_ADC_EnableSequencerChannel(ADC, channel);                  //通道选择

    FL_ADC_ClearFlag_EndOfConversion(ADC);                        //清标志
    FL_ADC_Enable(ADC);                                           //启动ADC
    FL_ADC_EnableSWConversion(ADC);                               //开始转换

    
    do
    {
        EOC_Flag = FL_ADC_IsActiveFlag_EndOfConversion(ADC);
        counter++;
    }while((counter != 0xFFFFFFFFU) && (EOC_Flag == 0U));           //等待转换完成         

    if(EOC_Flag == 0x01U)
    {    
      FL_ADC_ClearFlag_EndOfConversion(ADC);                        //清标志
      *ADCRdresult = FL_ADC_ReadConversionData(ADC);                 //获取采样值

      FL_ADC_Disable(ADC);                                          //关闭ADC
      FL_ADC_DisableSequencerChannel(ADC, channel);                 //通道关闭
      State = 0;
    }
    else
    {
      State = 1 ;
    }    
    return State;
}



uint32_t GetSingleChannelVoltage_POLL(uint32_t channel)
{
    uint32_t GetAVREFSample=0, GetChannelVoltage=0,GetVSample=0;  
    uint8_t  GetAVREFSample_State,GetVSample_State;  
  
    GetAVREFSample_State = GetAVREFSample_POLL(&GetAVREFSample);
    GetVSample_State = GetSingleChannelSample_POLL(channel,&GetVSample);
    
    if((GetAVREFSample != 0) &&(GetAVREFSample_State == 0)  &&(GetVSample_State == 0))
    {
      GetChannelVoltage = (uint32_t)(((uint64_t)GetVSample  * (ADC_VREF)) / ((uint64_t)GetAVREFSample * 10));  //计算通道电压
    }
    
    return GetChannelVoltage;
}

