#include "Tickless_Hook.h" 
#include "FreeRTOS.h"
#include "task.h"


#define   ulGetExternalTime()    FL_LPTIM32_ReadCounter(LPTIM32)


void sleepConfig(void)
{

    FL_PMU_SleepInitTypeDef    PMU_InitStruct;

    PMU_InitStruct.deepSleep = FL_PMU_SLEEP_MODE_DEEP;
    PMU_InitStruct.LDOLowPowerMode = FL_PMU_LDO_LPM_DISABLE;
    PMU_InitStruct.wakeupFrequency = FL_PMU_RCHF_WAKEUP_FREQ_8MHZ;
    PMU_InitStruct.wakeupDelay = FL_PMU_WAKEUP_DELAY_2US;
    PMU_InitStruct.coreVoltageScaling = FL_DISABLE;

    FL_PMU_Sleep_Init(PMU, &PMU_InitStruct);   
    
    FL_RMU_PDR_Enable(RMU);  			       //打开PDR
    FL_RMU_BORPowerDown_Disable(RMU);  	 //关闭BOR 2uA
       
    FL_PMU_SetLowPowerMode(PMU,FL_PMU_POWER_MODE_SLEEP_OR_DEEPSLEEP);	
    __WFI();
    
}
void prvStopTickInterruptTimer ( void )
{
    NVIC_DisableIRQ ( SysTick_IRQn );
    NVIC_ClearPendingIRQ ( SysTick_IRQn );
    SysTick->CTRL= 0;
}

__INLINE void disable_interrupts ( void )
{
    __asm volatile ( "cpsid i" );
    __asm volatile ( "dsb" );
    __asm volatile ( "isb" );
}

void prvStartTickInterruptTimer ( void )
{
    SysTick_Config ( configCPU_CLOCK_HZ / configTICK_RATE_HZ );
    NVIC_EnableIRQ ( SysTick_IRQn );
}

__INLINE void enable_interrupts ( void )
{
    __asm volatile ( "cpsie i" );
}

void prvSleep ( void )
{
    FL_LPTIM32_EnableIT_CC(LPTIM32,FL_LPTIM32_CHANNEL_1);       
    NVIC_DisableIRQ ( LPTIM_IRQn );
    NVIC_SetPriority ( LPTIM_IRQn, 2 );
    NVIC_EnableIRQ ( LPTIM_IRQn );
    sleepConfig (); 
    FL_LPTIM32_DisableIT_CC(LPTIM32,FL_LPTIM32_CHANNEL_1);           
}

void vSetWakeTimeInterrupt ( TickType_t xExpectedIdleTime )
{
    FL_LPTIM32_WriteCompareCH1(LPTIM32, xExpectedIdleTime + ulGetExternalTime()); 
}
  
void vApplicationSleep ( TickType_t xExpectedIdleTime )
{
    unsigned long ulLowPowerTimeBeforeSleep, ulLowPowerTimeAfterSleep;
    eSleepModeStatus eSleepStatus;
    FL_LPTIM32_Enable(LPTIM32); 
    /* Read the current time from a time source that will remain operational
    while the microcontroller is in a low power state. */
    ulLowPowerTimeBeforeSleep = ulGetExternalTime ();

    /* Stop the timer that is generating the tick interrupt. */
    prvStopTickInterruptTimer ();

    /* Enter a critical section that will not effect interrupts bringing the MCU
    out of sleep mode. */
    disable_interrupts ();

    /* Ensure it is still ok to enter the sleep mode. */
    eSleepStatus = eTaskConfirmSleepModeStatus ();

    if ( eSleepStatus == eAbortSleep )
    {
      /* A task has been moved out of the Blocked state since this macro was
      executed, or a context siwth is being held pending.  Do not enter a
      sleep state.  Restart the tick and exit the critical section. */
      prvStartTickInterruptTimer ();
      enable_interrupts ();
    }
    else 
    {
      if ( eSleepStatus == eNoTasksWaitingTimeout )
      {
        /* It is not necessary to configure an interrupt to bring the
        microcontroller out of its low power state at a fixed time in the
        future. */
        prvSleep ();
      }
      else 
      {

        /* Configure an interrupt to bring the microcontroller out of its low
        power state at the time the kernel next needs to execute.  The
        interrupt must be generated from a source that remains operational
        when the microcontroller is in a low power state. */
        vSetWakeTimeInterrupt ( xExpectedIdleTime - 1 );

        /* Enter the low power state. */
        prvSleep ();

        /* Determine how long the microcontroller was actually in a low power
        state for, which will be less than xExpectedIdleTime if the
        microcontroller was brought out of low power mode by an interrupt
        other than that configured by the vSetWakeTimeInterrupt() call.
        Note that the scheduler is suspended before
        portSUPPRESS_TICKS_AND_SLEEP() is called, and resumed when
        portSUPPRESS_TICKS_AND_SLEEP() returns.  Therefore no other tasks will
        execute until this function completes. */
        ulLowPowerTimeAfterSleep = ulGetExternalTime ();
        FL_LPTIM32_Disable(LPTIM32); 
        /* Correct the kernels tick count to account for the time the
        microcontroller spent in its low power state. */
        vTaskStepTick ( ( ulLowPowerTimeAfterSleep - ulLowPowerTimeBeforeSleep ) );
      }

      /* Exit the critical section - it might be possible to do this immediately
      after the prvSleep() calls. */
      enable_interrupts ();

      /* Restart the timer that is generating the tick interrupt. */
      prvStartTickInterruptTimer ();
    }
}
 
